<?php
/**
 * @brief		Admin CP Group Form
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	Rich Discord Integration
 * @since		03 Apr 2018
 */

namespace IPS\richdiscord\extensions\core\GroupForm;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Admin CP Group Form
 */
class _discordroles
{
	/**
	 * Process Form
     *
	 * @param	\IPS\Helpers\Form		$form	The form
	 * @param	\IPS\Member\Group		$group	Existing Group
	 * @return	void
	 */
	public function process( &$form, $group )
	{
	    if ($group->g_id == \IPS\Settings::i()->guest_group || !\IPS\Application::load('richdiscord')->isConfigured()) {
	        return;
        }
        $options = array();
	    try {
	        $roles = array_filter( \IPS\richdiscord\Discord\Guild::make( \IPS\Settings::i()->richdiscord_guild )->roles, function($val) {
                if ($val->id == \IPS\Settings::i()->richdiscord_guild || $val->managed) {
                    return FALSE;
                }
                return TRUE;
            } );
	        if (!count($roles)) return;
	        foreach ($roles as $role) {
	            $options[ $role->id ] = $role->name;
            }
        }
        catch ( \IPS\richdiscord\Discord\Exceptions\ForbiddenException $e )
        {
            \IPS\Log::log( 'Can\'t access guild. Maybe it was deleted?' );
            return;
        }
        catch ( \IPS\richdiscord\Discord\Exceptions\UnauthorizedException $e )
        {
            \IPS\Log::log( 'Outdated token. Please check your Discord settings.' );
            return;
        }
        catch ( \IPS\richdiscord\Discord\Exceptions\BaseException $e ) {return;}
		$form->add( new \IPS\Helpers\Form\Select( 'richdiscord_gform_roles', $group->g_richdiscord_roles ?: [], FALSE, [ 'multiple' => TRUE, 'options' => $options ] ) );
	}
	
	/**
	 * Save
	 *
	 * @param	array				$values	Values from form
	 * @param	\IPS\Member\Group	$group	The group
	 * @return	void
	 */
	public function save( $values, &$group )
	{
        if (!\IPS\Application::load('richdiscord')->isConfigured()) {
            return;
        }
		$group->g_richdiscord_roles = is_array($values['richdiscord_gform_roles']) ? implode(',', $values['richdiscord_gform_roles']) : $values['richdiscord_gform_roles'];
	}
}