<?php

namespace IPS\richdiscord\modules\admin\general;

use \IPS\richdiscord\Discord\Permissions;

/* To prevent PHP errors (extending class does not exist) revealing path */
if( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
    header( ( isset( $_SERVER[ 'SERVER_PROTOCOL' ] ) ? $_SERVER[ 'SERVER_PROTOCOL' ] : 'HTTP/1.0' ) . ' 403 Forbidden' );
    exit;
}

/**
 * configuration
 */
class _configuration extends \IPS\Dispatcher\Controller
{
    /**
     * Execute
     *
     * @return    void
     */
    public function execute()
    {
        \IPS\Dispatcher::i()->checkAcpPermission( 'configuration_manage' );
        parent::execute();
    }

    /**
     * ...
     *
     * @return    void
     */
    protected function manage()
    {
        \IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'menu__richdiscord_general_configuration' );
        $form                   = new \IPS\Helpers\Form;
        $addl                   = \IPS\Member::loggedIn()->language()->addToStack( 'richdiscord_add_following_callbacks' );
        $add2                   = \IPS\Member::loggedIn()->language()->addToStack( 'richdiscord_form_needed_perms' );
        $url2                   = \IPS\Settings::i()->base_url . 'applications/richdiscord/interface/oauth2/accounts.php';

        $form->addTab( 'richdiscord_tab_application' );
        $form->addHtml( <<<END
<div class="ipsMessage ipsMessage_information" style="margin: 15px">
    $addl
    <ul>
        <li>$url2</li>
    </ul>
    $add2
    <ul>
        <li>MANAGE_ROLES</li>
        <li>BAN_MEMBERS</li>
        <li>KICK_MEMBERS</li>
        <li>CREATE_INSTANT_INVITE</li>
        <li>MANAGE_NICKNAMES</li>
    </ul>
</div>
END
        );
        $form->add( new \IPS\Helpers\Form\Text( 'richdiscord_form_token', \IPS\Settings::i()->richdiscord_token, true, [], function( $val )
        {
            try
            {
                \IPS\richdiscord\Discord\Requestable::request( '/oauth2/applications/@me', 'GET', null, 'application/json', $val );
            }
            catch( \IPS\richdiscord\Discord\Exceptions\UnauthorizedException $e )
            {
                throw new \DomainException( 'richdiscord_invalid_token' );
            }
            catch( \IPS\richdiscord\Discord\Exceptions\BaseException $e )
            {
                throw new \DomainException( 'richdiscord_api_unavailable' );
            }
        } ) );
        $form->add( new \IPS\Helpers\Form\Text( 'richdiscord_form_client_id', \IPS\Settings::i()->richdiscord_client_id, true ) );
        $form->add( new \IPS\Helpers\Form\Text( 'richdiscord_form_secret', \IPS\Settings::i()->richdiscord_secret, true ) );
        $form->add( new \IPS\Helpers\Form\Text( 'richdiscord_form_guild_id', \IPS\Settings::i()->richdiscord_guild, true, [], function( $val )
        {
            if( !preg_match( "/^\d+$/", $val ) )
            {
                throw new \DomainException( "form_bad_value" );
            }
        } ) );
        $form->add( new \IPS\Helpers\Form\Text( 'richdiscord_form_invite_link', \IPS\Settings::i()->richdiscord_invite, false ) );

        if( \IPS\Application::appIsEnabled( 'forums' ) )
        {
            $form->add( new \IPS\Helpers\Form\YesNo( 'richdiscord_sendnotification', \IPS\Settings::i()->richdiscord_sendnotification, false, array( 'togglesOn' => array( 'richdiscord_forums', 'richdiscord_webhookurl' ) ) ) );
            $form->add( new \IPS\Helpers\Form\Node( 'richdiscord_forums', \IPS\Settings::i()->richdiscord_forums && \IPS\Settings::i()->richdiscord_forums != '0' ? explode( ',', \IPS\Settings::i()->richdiscord_forums ) : 0, false, array( 'class' => 'IPS\forums\Forum', 'multiple' => true, 'zeroVal' => 'all' ), null, null, null, 'richdiscord_forums' ) );
            $form->add( new \IPS\Helpers\Form\Url( 'richdiscord_webhookurl', \IPS\Settings::i()->richdiscord_webhookurl, false, array(), null, null, null, 'richdiscord_webhookurl' ) );
        }

        if( \IPS\Application::load( 'richdiscord' )->isConfigured() )
        {
            $form->addTab( 'richdiscord_tab_behavior' );
            $form->add( new \IPS\Helpers\Form\YesNo( 'richdiscord_form_enable_queue', \IPS\Settings::i()->richdiscord_enable_queue, false ) );
        }

        if( $values = $form->values() )
        {
            // Remove prefix
            foreach( $values as $k => $v )
            {
                if( mb_substr( $k, 0, 17 ) == 'richdiscord_form_' )
                {
                    unset( $values[ $k ] );
                    $values[ mb_substr( $k, 17 ) ] = $v;
                }
            }

            // Check if guild exists
            try
            {
                $response = \IPS\richdiscord\Discord\Requestable::request( '/users/@me/guilds', 'GET', null, 'application/json', $values[ 'token' ] )->decodeJson();
                $guild    = false;
                foreach( $response as $guildd )
                {
                    if( $guildd[ 'id' ] == trim( $values[ 'guild_id' ] ) )
                    {
                        $guild = $guildd;
                        break;
                    }
                }
                if( $guild === false )
                {
                    $currentTab                                                          = call_user_func( \Closure::bind( function()
                    {
                        return $this->currentTab;
                    }, $form, '\IPS\Helpers\Form' ) );
                    $form->elements[ $currentTab ][ 'richdiscord_form_guild_id' ]->error = \IPS\Member::loggedIn()->language()->addToStack( "richdiscord_invalid_guild" );
                    \IPS\Output::i()->output                                             = $form;

                    return;
                }
                $perms = Permissions::MANAGE_ROLES | Permissions::BAN_MEMBERS | Permissions::KICK_MEMBERS | Permissions::MANAGE_NICKNAMES | Permissions::CREATE_INSTANT_INVITE;
                if( !( ( $guild[ 'permissions' ] & Permissions::ADMINISTRATOR ) == Permissions::ADMINISTRATOR || ( $guild[ 'permissions' ] & $perms ) == $perms ) )
                {
                    $currentTab                                                          = call_user_func( \Closure::bind( function()
                    {
                        return $this->currentTab;
                    }, $form, '\IPS\Helpers\Form' ) );
                    $form->elements[ $currentTab ][ 'richdiscord_form_guild_id' ]->error = \IPS\Member::loggedIn()->language()->addToStack( "richdiscord_form_insufficient_perms" );
                    \IPS\Output::i()->output                                             = $form;

                    return;
                }
            }
                // Other exception, so probably API is unavailable
            catch( \IPS\richdiscord\Discord\Exceptions\BaseException $e )
            {
                $form->error             = \IPS\Member::loggedIn()->language()->addToStack( 'richdiscord_api_unavailable' );
                \IPS\Output::i()->output = $form;

                return;
            }

            $keys = [ 'invite_link' => 'invite', 'token' => 'token', 'guild_id' => 'guild', 'client_id' => 'client_id', 'secret' => 'secret' ];

            if( \IPS\Application::load( 'richdiscord' )->isConfigured() )
            {
                $keys[ 'enable_queue' ]   = 'enable_queue';
                $values[ 'enable_queue' ] = (int) (bool) $values[ 'enable_queue' ];
            }

            if( is_array( $values[ 'richdiscord_forums' ] ) && count( $values[ 'richdiscord_forums' ] ) )
            {
                $tmp = array();

                foreach( $values[ 'richdiscord_forums' ] as $richdiscordForum )
                {
                    if( is_object( $richdiscordForum ) )
                    {
                        $tmp[] = $richdiscordForum->_id;
                    }
                }

                if( count( $tmp ) )
                {
                    $values[ 'richdiscord_forums' ] = join( ',', $tmp );
                }
                else
                {
                    $values[ 'richdiscord_forums' ] = 0;
                }
            }
            else
            {
                $values[ 'richdiscord_forums' ] = 0;
            }

            foreach( $keys as $k => $v )
            {
                $tmp = $values[ $k ];
                unset( $values[ $k ] );
                $v            = 'richdiscord_' . $v;
                $values[ $v ] = $tmp;
            }

            $form->saveAsSettings( $values );

            \IPS\Output::i()->redirect( \IPS\Http\Url::internal( "app=richdiscord&module=general&controller=configuration" ), 'saved' );
        }

        $dispatcher = \IPS\Dispatcher::i();

        if( \IPS\Application::load( "richdiscord" )->isConfigured() && \IPS\Member::loggedIn()->hasAcpRestriction( $dispatcher->application, $dispatcher->module, 'configuration_manage' ) )
        {
            \IPS\Output::i()->sidebar[ 'actions' ][ 'clear' ] = array(
                'primary' => true,
                'icon'    => 'trash-o',
                'title'   => 'richdiscord_conf_delete_data',
                'link'    => \IPS\Http\Url::internal( "app=richdiscord&module=general&controller=configuration&do=clear" )->csrf(),
                'data'    => array( 'delete-discord-data' => '' ),
            );
        }

        \IPS\Output::i()->output = $form;
    }

    protected function clear()
    {
        \IPS\Session::i()->csrfCheck();

        \IPS\Application::load( 'richdiscord' )->clearDiscordSettings();
        if( !\IPS\Request::i()->isAjax() )
        {
            \IPS\Output::i()->redirect( \IPS\Http\Url::internal( "app=richdiscord&module=general&controller=configuration" ) );
        }
    }

    // Create new methods with the same name as the 'do' parameter which should execute it
}